import { world } from "@minecraft/server";
import { getWaystoneForId } from "./waystone_registry";
import { getDefaultTeam } from "./config";
import { GROUP_FAVORITES } from "./waystone_groups";
const playerTeamInitializedKey = "blay_waystones:team_initialized";
const playerActivatedWaystonesKey = "blay_waystones:activated_waystones";
const playerFavoriteWaystonesKey = "blay_waystones:favorite_waystones";
const playerTagTeamPrefix = "blay_waystones:team_";
export function addPlayerToWaystoneTeam(player, team) {
    player.addTag(playerTagTeamPrefix + team);
}
export function removePlayerFromWaystoneTeam(player, team) {
    player.removeTag(playerTagTeamPrefix + team);
}
export function initializeDefaultTeam(player) {
    const defaultTeam = getDefaultTeam();
    if (defaultTeam && !player.getDynamicProperty(playerTeamInitializedKey)) {
        player.setDynamicProperty(playerTeamInitializedKey, true);
        addPlayerToWaystoneTeam(player, defaultTeam);
    }
}
export function saveWaystoneActivation(player, waystone) {
    // On the first waystone activation, we make sure the player is joined into the default team
    initializeDefaultTeam(player);
    const activatedWaystones = getPlayerActivatedWaystoneIds(player);
    activatedWaystones.push(waystone.id);
    savePlayerActivatedWaystoneIds(player, activatedWaystones);
    for (const team of getPlayerWaystoneTeams(player)) {
        const teamActivatedWaystones = getTeamActivatedWaystoneIds(team);
        teamActivatedWaystones.push(waystone.id);
        saveTeamActivatedWaystoneIds(team, teamActivatedWaystones);
    }
}
export function isWaystoneActivated(player, waystone) {
    // Before querying the activation status of a waystone, we make sure the player is joined into the default team
    initializeDefaultTeam(player);
    return getPlayerActivatedWaystoneIds(player).indexOf(waystone.id) !== -1;
}
export function removeWaystoneActivations(waystone) {
    const teams = getWaystoneTeams();
    for (const team of teams) {
        const teamActivatedWaystones = getTeamActivatedWaystoneIds(team);
        const index = teamActivatedWaystones.indexOf(waystone.id);
        if (index !== -1) {
            teamActivatedWaystones.splice(index, 1);
            saveTeamActivatedWaystoneIds(team, teamActivatedWaystones);
        }
    }
}
export function getActivatedWaystones(player) {
    // Before building a list of activated waystones, we make sure the player is joined into the default team
    initializeDefaultTeam(player);
    const resultIds = getPlayerActivatedWaystoneIds(player);
    for (const team of getPlayerWaystoneTeams(player)) {
        resultIds.push(...getTeamActivatedWaystoneIds(team));
    }
    const uniqueResultIds = new Set(resultIds);
    const results = [];
    for (const id of uniqueResultIds) {
        const waystone = getWaystoneForId(id);
        if (waystone) {
            results.push(waystone);
        }
    }
    return results;
}
export function mapWaystonesByGroup(player, waystones) {
    const result = new Map();
    const favoriteIds = new Set(getPlayerFavoriteWaystoneIds(player));
    for (const waystone of waystones) {
        if (favoriteIds.has(waystone.id)) {
            const items = result.get(GROUP_FAVORITES) ?? [];
            items.push(waystone);
            result.set(GROUP_FAVORITES, items);
        }
        if (waystone.data.group) {
            const items = result.get(waystone.data.group) ?? [];
            items.push(waystone);
            result.set(waystone.data.group, items);
        }
    }
    return result;
}
export function isWaystoneFavorite(player, waystone) {
    const favoriteIds = getPlayerFavoriteWaystoneIds(player);
    return favoriteIds.indexOf(waystone.id) !== -1;
}
export function setWaystoneFavorite(player, waystone, favorite) {
    const favoriteIds = new Set(getPlayerFavoriteWaystoneIds(player));
    if (favorite) {
        favoriteIds.add(waystone.id);
    }
    else {
        favoriteIds.delete(waystone.id);
    }
    savePlayerFavoriteWaystoneIds(player, [...favoriteIds]);
}
export function getPlayerFavoriteWaystoneIds(player) {
    return parseWaystoneIds(player.getDynamicProperty(playerFavoriteWaystonesKey));
}
function savePlayerFavoriteWaystoneIds(player, waystoneIds) {
    player.setDynamicProperty(playerFavoriteWaystonesKey, JSON.stringify(waystoneIds));
}
export function getPlayerActivatedWaystoneIds(player) {
    return parseWaystoneIds(player.getDynamicProperty(playerActivatedWaystonesKey));
}
function savePlayerActivatedWaystoneIds(player, waystoneIds) {
    player.setDynamicProperty(playerActivatedWaystonesKey, JSON.stringify([...new Set(waystoneIds)]));
}
export function getPlayerActivatedWaystones(player) {
    return parseWaystones(player.getDynamicProperty(playerActivatedWaystonesKey));
}
function getWaystoneTeams() {
    return world
        .getDynamicPropertyIds()
        .filter((id) => id.startsWith("blay_waystones:team:") && id.endsWith(":activated_waystones"))
        .map((id) => id.substring("blay_waystones:team:".length, id.length - ":activated_waystones".length));
}
export function getPlayerWaystoneTeams(player) {
    const result = [];
    for (const tag of player.getTags()) {
        if (tag.startsWith(playerTagTeamPrefix)) {
            const team = tag.substring(playerTagTeamPrefix.length);
            result.push(team);
        }
    }
    return result;
}
export function getTeamActivatedWaystoneIds(team) {
    return parseWaystoneIds(world.getDynamicProperty(getTeamActivatedWaystonesKey(team)));
}
function saveTeamActivatedWaystoneIds(team, waystoneIds) {
    world.setDynamicProperty(getTeamActivatedWaystonesKey(team), JSON.stringify([...new Set(waystoneIds)]));
}
export function getTeamActivatedWaystones(team) {
    return parseWaystones(world.getDynamicProperty(getTeamActivatedWaystonesKey(team)));
}
function getTeamActivatedWaystonesKey(team) {
    return "blay_waystones:team:" + team + ":activated_waystones";
}
function parseWaystoneIds(json) {
    const result = [];
    if (typeof json === "string") {
        const waystoneIds = JSON.parse(json);
        for (const waystoneId of waystoneIds) {
            result.push(waystoneId);
        }
    }
    return result;
}
function parseWaystones(json) {
    const result = [];
    const waystoneIds = parseWaystoneIds(json);
    for (const waystoneId of waystoneIds) {
        const waystone = getWaystoneForId(waystoneId);
        if (waystone) {
            result.push(waystone);
        }
    }
    return result;
}

//# sourceMappingURL=../../_blay_waystonesDebug/player_persistence.js.map
