import { EntityComponentTypes, EquipmentSlot, ItemStack, } from "@minecraft/server";
import { damageItemInHand, getDistance, removeItemFromHand, returnItemToPlayer, vectorCenterH } from "./utils";
import { teleportToWaystone } from "./teleporter";
import { getActivatedWaystones } from "./player_persistence";
import { getItemBoundTo, setItemBoundTo } from "./bound_items";
import { showWaystoneSelection } from "./ui/ui_waystone_selection";
import { playRegularTeleportEffects } from "./teleport_effects";
import { getReturnPortal, hasSpaceForWarpPortal, spawnWarpPortalAround } from "./warp_portals";
import { getWaystoneForBlock } from "./waystone_registry";
export class WarpStoneComponent {
    onUse(event) {
        const player = event.source;
        player.dimension.spawnParticle("blay_waystones:scroll_read", player.location);
        doHoldUseHint(event.source);
    }
    async onCompleteUse(event) {
        const player = event.source;
        resetHoldUseHint(player);
        // onCompleteUse is called twice when you interact with the menu too fast - we prevent opening the menu twice with this cooldown
        if (player.getItemCooldown("blay_waystones:warp_stone") > 0) {
            return;
        }
        player.startItemCooldown("blay_waystones:warp_stone", 20);
        const selectedWaystone = await showWaystoneSelection(player, getActivatedWaystones(player));
        if (!player.isValid()) {
            return;
        }
        player.startItemCooldown("blay_waystones:warp_stone", 20);
        if (!selectedWaystone) {
            return;
        }
        teleportToWaystone(player, selectedWaystone);
        playRegularTeleportEffects(player, selectedWaystone);
        if (event.itemStack) {
            damageItemInHand(player, event.itemStack, 1);
        }
    }
}
export class BlankScrollComponent {
    onUseOn(event) {
        const target = getWaystoneForBlock(event.block);
        if (target && tryBindBlankScroll(event.source, target)) {
            return;
        }
    }
}
export class BoundScrollComponent {
    onUse(event) {
        const player = event.source;
        player.dimension.spawnParticle("blay_waystones:scroll_read", player.location);
        doHoldUseHint(event.source);
    }
    onCompleteUse(event) {
        const player = event.source;
        resetHoldUseHint(player);
        removeItemFromHand(player, event.itemStack, 1);
        const boundTo = getItemBoundTo(event.itemStack);
        if (boundTo) {
            teleportToWaystone(player, boundTo);
            playRegularTeleportEffects(player, boundTo);
        }
        else {
            const blankScroll = new ItemStack("blay_waystones:blank_scroll", 1);
            returnItemToPlayer(player, blankScroll);
            player.sendMessage({ translate: "chat.blay_waystones.scroll_attunement_lost" });
        }
    }
}
export class WarpScrollComponent {
    onUse(event) {
        const player = event.source;
        player.dimension.spawnParticle("blay_waystones:scroll_read", player.location);
        doHoldUseHint(event.source);
    }
    async onCompleteUse(event) {
        const player = event.source;
        resetHoldUseHint(player);
        // onCompleteUse is called twice when you interact with the menu too fast - we prevent opening the menu twice with this cooldown
        if (player.getItemCooldown("blay_waystones:warp_scroll") > 0) {
            return;
        }
        player.startItemCooldown("blay_waystones:warp_scroll", 20);
        const selectedWaystone = await showWaystoneSelection(player, getActivatedWaystones(player));
        if (!player.isValid()) {
            return;
        }
        player.startItemCooldown("blay_waystones:warp_scroll", 20);
        if (!selectedWaystone) {
            return;
        }
        teleportToWaystone(player, selectedWaystone);
        playRegularTeleportEffects(player, selectedWaystone);
        if (event.itemStack) {
            removeItemFromHand(player, event.itemStack, 1);
        }
    }
}
export class PortalScrollComponent {
    onUse(event) {
        const player = event.source;
        player.dimension.spawnParticle("blay_waystones:scroll_read", player.location);
        doHoldUseHint(event.source);
    }
    async onCompleteUse(event) {
        const player = event.source;
        resetHoldUseHint(player);
        // onCompleteUse is called twice when you interact with the menu too fast - we prevent opening the menu twice with this cooldown
        if (player.getItemCooldown("blay_waystones:portal_scroll") > 0) {
            return;
        }
        if (!hasSpaceForWarpPortal(player)) {
            player.sendMessage({ translate: "chat.blay_waystones.warp_portal.no_space" });
            return;
        }
        player.startItemCooldown("blay_waystones:portal_scroll", 20);
        const selectedWaystone = await showWaystoneSelection(player, getActivatedWaystones(player), {
            disallowReturnPortal: true,
        });
        if (!player.isValid()) {
            return;
        }
        player.startItemCooldown("blay_waystones:portal_scroll", 20);
        if (!selectedWaystone) {
            return;
        }
        if (!spawnWarpPortalAround(player, selectedWaystone)) {
            player.sendMessage({ translate: "chat.blay_waystones.warp_portal.no_space" });
            return;
        }
        if (event.itemStack) {
            removeItemFromHand(player, event.itemStack, 1);
        }
    }
}
export class ReturnScrollComponent {
    onUse(event) {
        const player = event.source;
        player.dimension.spawnParticle("blay_waystones:scroll_read", player.location);
        doHoldUseHint(event.source);
    }
    onCompleteUse(event) {
        const player = event.source;
        resetHoldUseHint(player);
        const nearestWaystone = pickReturnTarget(player);
        if (!nearestWaystone) {
            player.sendMessage({ translate: "chat.blay_waystones.no_waystones_available" });
            return;
        }
        teleportToWaystone(player, nearestWaystone);
        playRegularTeleportEffects(player, nearestWaystone);
        removeItemFromHand(player, event.itemStack, 1);
    }
}
function pickReturnTarget(player) {
    const returnPortal = getReturnPortal(player);
    if (returnPortal) {
        return returnPortal;
    }
    const activatedWaystones = getActivatedWaystones(player);
    const sameDimensionWaystones = activatedWaystones.filter((waystone) => waystone.dimensionId === player.dimension.id);
    let nearestWaystone = null;
    let nearestDistance = Number.MAX_VALUE;
    for (const waystone of sameDimensionWaystones) {
        const distance = getDistance(player.location, waystone.pos);
        if (distance < nearestDistance) {
            nearestWaystone = waystone;
            nearestDistance = distance;
        }
    }
    return nearestWaystone;
}
export function tryBindBlankScroll(entity, waystone) {
    const equippable = entity.getComponent(EntityComponentTypes.Equippable);
    if (!equippable) {
        return false;
    }
    const mainHand = equippable.getEquipment(EquipmentSlot.Mainhand);
    if (mainHand?.typeId !== "blay_waystones:blank_scroll") {
        return false;
    }
    removeItemFromHand(entity, mainHand, 1);
    const itemStack = new ItemStack("blay_waystones:bound_scroll", 1);
    setItemBoundTo(itemStack, waystone);
    returnItemToPlayer(entity, itemStack);
    entity.dimension.spawnParticle("blay_waystones:scroll_bound", entity.location);
    entity.dimension.spawnParticle("blay_waystones:waystone_activation", vectorCenterH(waystone.pos));
    return true;
}
function doHoldUseHint(player) {
    let counter = player.getDynamicProperty("blay_waystones:hold_use_hint_counter") ?? 0;
    counter++;
    if (counter > 3) {
        player.sendMessage({ translate: "chat.blay_waystones.hold_use_hint", with: [":_input_key.use:"] });
        player.setDynamicProperty("blay_waystones:hold_use_hint_counter", -3);
    }
    else {
        player.setDynamicProperty("blay_waystones:hold_use_hint_counter", counter);
    }
}
function resetHoldUseHint(player) {
    player.setDynamicProperty("blay_waystones:hold_use_hint_counter", -3);
}

//# sourceMappingURL=../../_blay_waystonesDebug/scrolls.js.map
