import { world } from "@minecraft/server";
import { getWaystoneForBlock, getWaystoneIdKey, getWaystoneType, removeWaystoneAt, saveWaystoneData, } from "./waystone_registry";
import { teleportToWaystone } from "./teleporter";
import { showSharestoneSetup } from "./ui/ui_waystone_setup";
import { showSharestoneSelection } from "./ui/ui_waystone_selection";
import { playRegularTeleportEffects } from "./teleport_effects";
import { tryBindBlankScroll } from "./scrolls";
export class SharestoneTopComponent {
    onPlayerInteract(event) {
        const player = event.player;
        if (player) {
            // The waystone is registered to the base of the waystone double block, so we delegate
            let baseBlock = event.block?.below();
            if (baseBlock?.hasTag("blay_waystones:sharestone")) {
                const sharestoneType = getWaystoneType(baseBlock);
                const sharestone = getWaystoneForBlock(baseBlock);
                if (sharestoneType && sharestone) {
                    interactSharestone(player, sharestoneType, sharestone);
                }
            }
        }
    }
    onPlayerDestroy(event) {
        destroySharestoneBase(event.dimension, event.block.below());
    }
}
export class SharestoneComponent {
    beforeOnPlayerPlace(event) {
        // Only allow placement if the block above is free
        const above = event.block.above();
        if (!above?.isAir) {
            event.cancel = true;
        }
    }
    onPlace(event) {
        const above = event.block.above();
        if (above?.isAir) {
            above.setPermutation(event.block.permutation.withState("blay_waystones:half", "top"));
        }
    }
    // Post-placement behaviour is done in main due to missing player in BlockComponentPlaceEvent
    onPlayerInteract(event) {
        const player = event.player;
        if (player) {
            const sharestoneType = getWaystoneType(event.block);
            const sharestone = getWaystoneForBlock(event.block);
            if (sharestoneType && sharestone) {
                interactSharestone(player, sharestoneType, sharestone);
            }
        }
    }
    onPlayerDestroy(event) {
        removeWaystoneAt(event.dimension.id, event.block.location);
    }
}
function getSharestoneKeyPrefix(type) {
    return "blay_sharestones:type:" + type + ":";
}
function isSharestoneKey(type, propertyId) {
    return propertyId.startsWith(getSharestoneKeyPrefix(type));
}
function getSharestoneKey(type, id) {
    return getSharestoneKeyPrefix(type) + id;
}
export function getSharestones(type) {
    const result = [];
    for (const propertyId of world.getDynamicPropertyIds()) {
        if (isSharestoneKey(type, propertyId)) {
            const sharestoneId = world.getDynamicProperty(propertyId);
            if (typeof sharestoneId !== "string") {
                continue;
            }
            const idKey = getWaystoneIdKey(sharestoneId);
            const sharestoneJson = world.getDynamicProperty(idKey);
            if (typeof sharestoneJson === "string") {
                const sharestone = JSON.parse(sharestoneJson);
                if (sharestone.pos && sharestone.dimensionId) {
                    result.push({
                        id: sharestoneId,
                        pos: sharestone.pos,
                        dimensionId: sharestone.dimensionId,
                        data: sharestone,
                    });
                }
            }
        }
    }
    return result;
}
export function removeSharestoneAt(dimensionId, pos) {
    const waystone = removeWaystoneAt(dimensionId, pos);
    if (waystone && waystone.data.type) {
        const sharestoneKey = getSharestoneKey(waystone.data.type, waystone.id);
        world.setDynamicProperty(sharestoneKey, undefined);
    }
}
export function saveSharestoneData(waystone) {
    if (waystone.data.type) {
        saveWaystoneData(waystone);
        const sharestoneKey = getSharestoneKey(waystone.data.type, waystone.id);
        world.setDynamicProperty(sharestoneKey, waystone.id);
    }
}
export async function interactSharestone(player, sharestoneType, sharestone) {
    // Block interaction takes precedence, so we have to trigger scroll binding from here
    if (tryBindBlankScroll(player, sharestone)) {
        return;
    }
    if (player.isSneaking) {
        showSharestoneSetup(player, sharestone);
    }
    else {
        const selectedWaystone = await showSharestoneSelection(player, getSharestones(sharestoneType), {
            fromWaystone: sharestone,
        });
        if (selectedWaystone && player.isValid()) {
            teleportToWaystone(player, selectedWaystone);
            playRegularTeleportEffects(player, selectedWaystone);
        }
    }
}
export function destroySharestoneBase(dimension, block) {
    if (block?.hasTag("blay_waystones:sharestone")) {
        dimension.runCommand("setblock " + block.x + " " + block.y + " " + block.z + " air destroy");
        removeSharestoneAt(dimension.id, block.location);
    }
}

//# sourceMappingURL=../../_blay_waystonesDebug/sharestones.js.map
