import { getPlayerFavoriteWaystoneIds, mapWaystonesByGroup } from "../player_persistence";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";
import { allGroups, groupIcons } from "../waystone_groups";
import { getReturnPortal } from "../warp_portals";
export async function showSharestoneSelection(player, waystones, options) {
    return await showWaystoneSelection(player, waystones, {
        title: { translate: "gui.blay_waystones.sharestone_selection.title" },
        noTargetsAvailableMessage: { translate: "chat.blay_waystones.no_sharestones_available" },
        disallowReturnPortal: true,
        ...options,
    });
}
function sortWaystones(a, b) {
    const nameA = a.data.name?.toLowerCase() ?? "";
    const nameB = b.data.name?.toLowerCase() ?? "";
    return nameA.localeCompare(nameB);
}
export async function showWaystoneSelection(player, waystones, options) {
    const returnPortal = getReturnPortal(player);
    const showReturnPortal = returnPortal && !options?.disallowReturnPortal;
    if (options?.fromWaystone) {
        waystones = waystones.filter((waystone) => waystone.id !== options.fromWaystone.id);
    }
    if (!waystones.length && !showReturnPortal) {
        player.sendMessage(options?.noTargetsAvailableMessage ?? { translate: "chat.blay_waystones.no_waystones_available" });
        return undefined;
    }
    const form = new ActionFormData().title(options?.title ?? { translate: "gui.blay_waystones.waystone_selection.title" });
    const defaultMax = 50;
    const defaultInitialChoices = [];
    if (waystones.length > 5) {
        defaultInitialChoices.push({
            type: "find",
            label: { translate: "gui.blay_waystones.waystone_selection.find_waystone" },
        });
    }
    if (showReturnPortal) {
        defaultInitialChoices.push({
            type: "waystone",
            label: { translate: "gui.blay_waystones.waystone_selection.return_to_portal" },
            value: returnPortal,
        });
    }
    const choices = options?.initialChoices ?? defaultInitialChoices;
    const effectiveFormatter = options?.waystoneFormatter ?? ((waystone) => waystone.data.name);
    const favoriteWaystoneIds = new Set(getPlayerFavoriteWaystoneIds(player));
    const favoriteWaystones = waystones.filter((waystone) => favoriteWaystoneIds.has(waystone.id)).sort(sortWaystones);
    const nonFavoriteWaystones = waystones
        .filter((waystone) => !favoriteWaystoneIds.has(waystone.id))
        .sort(sortWaystones);
    const sortedWaystones = [...favoriteWaystones, ...nonFavoriteWaystones];
    const limitedWaystones = sortedWaystones.slice(options?.offset ?? 0, (options?.offset ?? 0) + (options?.max ?? defaultMax));
    for (const waystone of limitedWaystones) {
        choices.push({
            label: effectiveFormatter(waystone) ?? { translate: "gui.blay_waystones.waystone_selection.missingno" },
            type: "waystone",
            icon: waystone.data.group ? groupIcons.get(waystone.data.group) : undefined,
            value: waystone,
        });
    }
    if ((options?.offset ?? 0) + (options?.max ?? defaultMax) < sortedWaystones.length) {
        choices.push({
            label: { translate: "gui.blay_waystones.waystone_selection.show_more" },
            type: "more",
        });
    }
    const visibleChoices = choices.filter((it) => it.condition === undefined || it.condition);
    for (const choice of visibleChoices) {
        form.button(choice.label, choice.icon);
    }
    const response = await form.show(player);
    if (response.canceled || response.selection === undefined || !player.isValid()) {
        return undefined;
    }
    const selectedChoice = visibleChoices[response.selection];
    if (selectedChoice.type === "find") {
        const groupedWaystones = mapWaystonesByGroup(player, waystones);
        if (!groupedWaystones.size) {
            return await showWaystoneSearch(player, options?.originalWaystones ?? waystones, {
                ...options,
                fromWaystone: options?.fromWaystone,
            });
        }
        else {
            return await showWaystoneFinder(player, options?.originalWaystones ?? waystones, {
                ...options,
                fromWaystone: options?.fromWaystone,
            });
        }
    }
    else if (selectedChoice.type === "search") {
        return await showWaystoneSearch(player, options?.originalWaystones ?? waystones, {
            ...options,
            fromWaystone: options?.fromWaystone,
        });
    }
    else if (selectedChoice.type === "waystone") {
        return selectedChoice.value;
    }
    else if (selectedChoice.type === "more") {
        return await showWaystoneSelection(player, options?.originalWaystones ?? waystones, {
            ...options,
            offset: (options?.offset ?? 0) + limitedWaystones.length,
        });
    }
    return undefined;
}
async function showWaystoneFinder(player, waystones, options) {
    const groupedWaystones = mapWaystonesByGroup(player, waystones);
    const form = new ActionFormData().title(options?.title ?? { translate: "gui.blay_waystones.waystone_selection.title" });
    const choices = [
        {
            type: "search",
            label: { translate: "gui.blay_waystones.waystone_selection.search_by_name" },
        },
    ];
    for (const group of allGroups) {
        const groupItems = groupedWaystones.get(group.id);
        choices.push({
            label: group.label,
            type: "group",
            icon: group.icon,
            value: groupItems,
            condition: (groupItems?.length ?? 0) > 0,
        });
    }
    const visibleChoices = choices.filter((it) => it.condition === undefined || it.condition);
    for (const choice of visibleChoices) {
        form.button(choice.label, choice.icon);
    }
    const response = await form.show(player);
    if (response.canceled || response.selection === undefined || !player.isValid()) {
        return undefined;
    }
    const selectedChoice = visibleChoices[response.selection];
    if (selectedChoice.type === "search") {
        return await showWaystoneSearch(player, waystones, { ...options, fromWaystone: options?.fromWaystone });
    }
    else if (selectedChoice.type === "group") {
        return await showWaystoneSelection(player, selectedChoice.value, {
            ...options,
            fromWaystone: options?.fromWaystone,
            title: selectedChoice.label,
            initialChoices: [
                {
                    type: "find",
                    label: { translate: "gui.blay_waystones.waystone_selection.back" },
                },
            ],
            originalWaystones: options?.originalWaystones ?? waystones,
        });
    }
}
async function showWaystoneSearch(player, waystones, options) {
    const modalForm = new ModalFormData().title({ translate: "gui.blay_waystones.waystone_search.title" });
    modalForm.textField(options?.body ?? "", { translate: "gui.blay_waystones.waystone_search.placeholder" }, "");
    try {
        const formData = await modalForm.show(player);
        if (formData.canceled || !player.isValid()) {
            return undefined;
        }
        const searchTerm = formData.formValues?.[0];
        if (!searchTerm) {
            return await showWaystoneSearch(player, waystones, {
                ...options,
                body: { translate: "gui.blay_waystones.waystone_search.no_results" },
            });
        }
        const filteredWaystones = waystones.filter((it) => it.data.name?.toLowerCase().includes(searchTerm.toLowerCase()));
        // const searchTermPattern = new RegExp(regexEscape(searchTerm), "gi");
        if (filteredWaystones.length) {
            return await showWaystoneSelection(player, filteredWaystones, {
                ...options,
                fromWaystone: options?.fromWaystone,
                title: { translate: "gui.blay_waystones.waystone_search_results.title" },
                initialChoices: [
                    {
                        type: "search",
                        label: { translate: "gui.blay_waystones.waystone_search_results.back_to_search" },
                    },
                ],
                originalWaystones: options?.originalWaystones ?? waystones,
                // We don't highlight matches because §r does not reset to the correct color in buttons and their front color changes on hover too
                // waystoneFormatter: (waystone) => waystone.data.name?.replace(searchTermPattern, `§2$&§r`),
            });
        }
        else {
            return await showWaystoneSearch(player, waystones, {
                ...options,
                body: { translate: "gui.blay_waystones.waystone_search.no_results" },
            });
        }
    }
    catch (error) {
        player.sendMessage("Failed to show form: " + error);
    }
    return undefined;
}

//# sourceMappingURL=../../../_blay_waystonesDebug/ui/ui_waystone_selection.js.map
