import { ItemStack, world, system, } from "@minecraft/server";
import { getWaystoneForBlock, getWaystoneForId, removeWaystoneAt, saveWaystoneData } from "./waystone_registry";
import { teleportToWaystone } from "./teleporter";
import { isTeleportAllowed, removeItemFromHand, vectorCenter, vectorCenterH } from "./utils";
import { generateName } from "./name_generator";
import { getItemBoundTo, setItemBoundTo } from "./bound_items";
import { playWarpPlateTeleportEffects } from "./teleport_effects";
import { tryBindBlankScroll } from "./scrolls";
export class DormantShardComponent {
    onUseOn(event) {
        if (isEmptyWarpPlate(event.usedOnBlockPermutation)) {
            const shardTint = event.usedOnBlockPermutation.getState("blay_waystones:shard_tint");
            event.block.setPermutation(event.usedOnBlockPermutation
                .withState("blay_waystones:state", "attuning")
                .withState("blay_waystones:tint", shardTint));
            event.source.dimension.playSound("random.pop2", event.block.location);
            event.source.dimension.spawnParticle("blay_waystones:warp_plate_attuning", vectorCenterH(event.block.location));
            removeItemFromHand(event.source, event.itemStack);
        }
    }
}
export class AttunedShardComponent {
    onUseOn(event) {
        if (isEmptyWarpPlate(event.usedOnBlockPermutation)) {
            const warpPlate = getOrCreateWarpPlate(event.block);
            const attunedTo = getItemBoundTo(event.itemStack);
            const shardTint = event.usedOnBlockPermutation.getState("blay_waystones:shard_tint");
            if (!attunedTo) {
                // If the warp plate this shard was attuned to is no longer valid, break the shard
                event.source.dimension.playSound("random.break", event.block.location);
                removeItemFromHand(event.source, event.itemStack);
                if (event.source.typeId === "minecraft:player") {
                    event.source.sendMessage({ translate: "chat.blay_waystones.shard_attunement_lost" });
                }
            }
            else if (attunedTo?.id === warpPlate?.id) {
                // If the shard was attuned to this warp plate, reject it
                event.source.dimension.spawnParticle("blay_waystones:warp_plate_smoke", vectorCenterH(event.block.location));
                event.source.dimension.playSound("scrape", event.block.location);
                if (event.source.typeId === "minecraft:player") {
                    event.source.sendMessage({ translate: "chat.blay_waystones.warp_plate.reject_own_shard" });
                }
            }
            else {
                const tint = getTintForAttunedShardType(event.itemStack.typeId);
                event.block.setPermutation(event.usedOnBlockPermutation
                    .withState("blay_waystones:state", "attuned")
                    .withState("blay_waystones:tint", tint));
                setBlockBoundTo(event.block, attunedTo);
                event.source.dimension.playSound("random.pop2", event.block.location);
                removeItemFromHand(event.source, event.itemStack);
            }
        }
    }
}
export class WarpPlateEmptyComponent {
    beforeOnPlayerPlace(event) {
        const tint = getNextWarpPlateTint(event.player);
        event.permutationToPlace = event.permutationToPlace
            .withState("blay_waystones:state", "attuning")
            .withState("blay_waystones:tint", tint)
            .withState("blay_waystones:shard_tint", tint);
    }
    onStepOn(event) {
        if (event.entity?.typeId !== "minecraft:item") {
            event.dimension.spawnParticle("blay_waystones:warp_plate_smoke", vectorCenterH(event.block.location));
        }
    }
    onStepOff(event) {
        tryClearWarpPlateEntityCooldown(event.entity, event.block);
    }
    onPlayerDestroy(event) {
        removeWarpPlateAt(event.dimension.id, event.block.location);
    }
}
export class WarpPlateAttuningComponent {
    onPlace(event) {
        event.dimension.spawnParticle("blay_waystones:warp_plate_attuning", vectorCenterH(event.block.location));
    }
    onPlayerInteract(event) {
        if (event.player?.isSneaking) {
            event.dimension.spawnParticle("blay_waystones:warp_plate_smoke", vectorCenterH(event.block.location));
            event.block.setPermutation(event.block.permutation.withState("blay_waystones:state", "empty"));
            event.dimension.spawnItem(new ItemStack("blay_waystones:dormant_shard", 1), vectorCenter(event.block.location));
            event.dimension.playSound("random.pop", event.block.location);
        }
    }
    onTick(event) {
        const shardTint = event.block.permutation.getState("blay_waystones:shard_tint");
        event.block.setPermutation(event.block.permutation.withState("blay_waystones:state", "empty"));
        const warpPlate = getOrCreateWarpPlate(event.block);
        if (warpPlate) {
            const itemType = getAttunedShardTypeForTint(shardTint);
            const itemStack = new ItemStack(itemType, 1);
            setItemBoundTo(itemStack, warpPlate);
            event.dimension.spawnItem(itemStack, vectorCenter(event.block.location));
            event.dimension.playSound("random.pop", event.block.location);
            event.dimension.spawnParticle("blay_waystones:warp_plate_attuned", vectorCenterH(event.block.location));
        }
        else {
            event.dimension.spawnItem(new ItemStack("blay_waystones:dormant_shard", 1), vectorCenter(event.block.location));
            event.dimension.spawnParticle("blay_waystones:warp_plate_smoke", vectorCenterH(event.block.location));
            event.dimension.playSound("random.pop", event.block.location);
        }
    }
    onStepOff(event) {
        tryClearWarpPlateEntityCooldown(event.entity, event.block);
    }
    onPlayerDestroy(event) {
        removeWarpPlateAt(event.dimension.id, event.block.location);
    }
}
export class WarpPlateAttunedComponent {
    onPlayerInteract(event) {
        const warpPlate = getWaystoneForBlock(event.block);
        // Block interaction takes precedence, so we have to trigger scroll binding from here
        if (event.player && warpPlate && tryBindBlankScroll(event.player, warpPlate)) {
            return;
        }
        if (event.player?.isSneaking) {
            dropContainedShard(event.block, event.block.permutation);
            event.block.setPermutation(event.block.permutation.withState("blay_waystones:state", "empty"));
        }
    }
    onStepOn(event) {
        const warpPlate = getWaystoneForBlock(event.block);
        if (event.entity && warpPlate && !isWarpPlateEntityCooldown(event.entity, warpPlate)) {
            const boundTo = getBlockBoundTo(event.block);
            if (boundTo) {
                if (isTeleportAllowed(event.entity, boundTo)) {
                    event.dimension.spawnParticle("blay_waystones:warp_plate_charging", vectorCenterH(event.block.location));
                    event.dimension.playSound("portal.trigger", event.block.location, {
                        volume: 0.25,
                    });
                    event.block.setPermutation(event.block.permutation.withState("blay_waystones:state", "warping"));
                }
            }
            else {
                event.dimension.spawnParticle("blay_waystones:warp_plate_smoke", vectorCenterH(event.block.location));
                event.dimension.playSound("random.break", event.block.location);
                event.block.setPermutation(event.block.permutation.withState("blay_waystones:state", "empty"));
                if (event.entity.typeId === "minecraft:player") {
                    event.entity.sendMessage({ translate: "chat.blay_waystones.shard_attunement_lost" });
                }
            }
        }
    }
    onStepOff(event) {
        tryClearWarpPlateEntityCooldown(event.entity, event.block);
    }
    onPlayerDestroy(event) {
        dropContainedShard(event.block, event.destroyedBlockPermutation);
        removeWarpPlateAt(event.dimension.id, event.block.location);
    }
}
export class WarpPlateWarpingComponent {
    onTick(event) {
        event.block.setPermutation(event.block.permutation.withState("blay_waystones:state", "attuned"));
        const warpPlate = getOrCreateWarpPlate(event.block);
        const boundTo = getBlockBoundTo(event.block);
        if (warpPlate && boundTo) {
            let teleportedAtLeastOne = false;
            const entities = event.block.dimension.getEntitiesAtBlockLocation(event.block);
            for (const entity of entities) {
                if (!isWarpPlateEntityCooldown(entity, warpPlate) && isTeleportAllowed(entity, boundTo)) {
                    teleportedAtLeastOne = true;
                    setWarpPlateEntityCooldown(entity, boundTo);
                    teleportToWaystone(entity, boundTo);
                }
            }
            if (teleportedAtLeastOne) {
                playWarpPlateTeleportEffects(warpPlate, boundTo);
            }
        }
    }
    onStepOff(event) {
        tryClearWarpPlateEntityCooldown(event.entity, event.block);
        // we double-check it's valid because it looks like stepOff can be called on an unloaded chunk when the player teleports away
        if (event.block.isValid()) {
            const entities = event.block.dimension.getEntitiesAtBlockLocation(event.block);
            if (!entities.length) {
                event.block.setPermutation(event.block.permutation.withState("blay_waystones:state", "attuned"));
            }
        }
    }
    onPlayerDestroy(event) {
        dropContainedShard(event.block, event.destroyedBlockPermutation);
        removeWarpPlateAt(event.dimension.id, event.block.location);
    }
}
const entityCooldownKey = "blay_waystones:warp_plate_cooldown_source";
export function dropContainedShard(block, permutation) {
    const attunedTo = getBlockBoundTo(block);
    if (attunedTo) {
        const shardTint = permutation.getState("blay_waystones:tint");
        const itemType = getAttunedShardTypeForTint(shardTint);
        const itemStack = new ItemStack(itemType, 1);
        setItemBoundTo(itemStack, attunedTo);
        block.dimension.spawnItem(itemStack, vectorCenter(block.location));
        block.dimension.playSound("random.pop", block.location);
    }
    else {
        const itemStack = new ItemStack("blay_waystones:dormant_shard", 1);
        block.dimension.spawnItem(itemStack, vectorCenter(block.location));
        block.dimension.playSound("random.pop", block.location);
    }
}
function isWarpPlateEntityCooldown(entity, warpPlate) {
    const jsonString = entity.getDynamicProperty(entityCooldownKey);
    if (typeof jsonString === "string") {
        const jsonData = JSON.parse(jsonString);
        if (jsonData.warpPlateId === warpPlate.id) {
            if (jsonData.expiresAt) {
                if (jsonData.expiresAt <= system.currentTick) {
                    clearWarpPlateEntityCooldown(entity);
                    return false;
                }
            }
            return true;
        }
    }
    return false;
}
function setWarpPlateEntityCooldown(entity, warpPlate) {
    if (entity.typeId === "minecraft:player") {
        entity.setDynamicProperty(entityCooldownKey, JSON.stringify({
            warpPlateId: warpPlate.id,
        }));
    }
    else if (entity.typeId === "minecraft:item") {
        entity.setDynamicProperty(entityCooldownKey, JSON.stringify({
            warpPlateId: warpPlate.id,
        }));
    }
    else {
        entity.setDynamicProperty(entityCooldownKey, JSON.stringify({
            warpPlateId: warpPlate.id,
            expiresAt: system.currentTick + 100,
        }));
    }
}
function tryClearWarpPlateEntityCooldown(entity, block) {
    if (entity?.isValid() && block?.isValid() && entity?.typeId === "minecraft:player") {
        const warpPlate = getOrCreateWarpPlate(block);
        if (warpPlate && isWarpPlateEntityCooldown(entity, warpPlate)) {
            clearWarpPlateEntityCooldown(entity);
        }
    }
}
function clearWarpPlateEntityCooldown(entity) {
    entity.setDynamicProperty(entityCooldownKey, undefined);
}
const warpPlateBoundToKeyPrefix = "blay_waystones:warp_plate:";
export function getWarpPlateBoundToKey(id) {
    return warpPlateBoundToKeyPrefix + id;
}
function getBlockBoundTo(block) {
    const warpPlate = getOrCreateWarpPlate(block);
    if (warpPlate) {
        const warpPlateKey = getWarpPlateBoundToKey(warpPlate.id);
        const waystoneId = world.getDynamicProperty(warpPlateKey);
        if (typeof waystoneId === "string") {
            return getWaystoneForId(waystoneId);
        }
    }
    return undefined;
}
function setBlockBoundTo(block, waystone) {
    const warpPlate = getOrCreateWarpPlate(block);
    if (warpPlate) {
        const warpPlateKey = getWarpPlateBoundToKey(warpPlate.id);
        if (waystone) {
            world.setDynamicProperty(warpPlateKey, waystone.id);
        }
        else {
            world.setDynamicProperty(warpPlateKey, undefined);
        }
    }
}
export function removeWarpPlateAt(dimensionId, pos) {
    const waystone = removeWaystoneAt(dimensionId, pos);
    if (waystone) {
        const warpPlateKey = getWarpPlateBoundToKey(waystone.id);
        world.setDynamicProperty(warpPlateKey, undefined);
    }
}
function getOrCreateWarpPlate(block) {
    const warpPlate = getWaystoneForBlock(block);
    if (warpPlate) {
        if (!warpPlate.data.name) {
            warpPlate.data.name = generateName();
        }
        saveWaystoneData(warpPlate);
    }
    return warpPlate;
}
const warpPlateTints = [
    "magenta",
    "blue",
    "brown",
    "cyan",
    "green",
    "light_blue",
    "lime",
    "orange",
    "pink",
    "purple",
    "red",
    "yellow",
];
function getNextWarpPlateTint(player) {
    if (!player) {
        return warpPlateTints[Math.floor(Math.random() * warpPlateTints.length)];
    }
    const storedLastTintIndex = player.getDynamicProperty("blay_waystones:last_warp_plate_tint");
    const lastTintIndex = typeof storedLastTintIndex === "number" ? storedLastTintIndex : -1;
    const nextTintIndex = (lastTintIndex + 1) % warpPlateTints.length;
    player.setDynamicProperty("blay_waystones:last_warp_plate_tint", nextTintIndex);
    return warpPlateTints[nextTintIndex];
}
function getAttunedShardTypeForTint(tint) {
    return "blay_waystones:" + tint + "_attuned_shard";
}
function getTintForAttunedShardType(type) {
    return type.replace("blay_waystones:", "").replace("_attuned_shard", "");
}
function isEmptyWarpPlate(permutation) {
    return permutation.matches("blay_waystones:warp_plate") && permutation.getState("blay_waystones:state") === "empty";
}
export function sneakHijackInteractWarpPlate(player, block) {
    if (block.permutation.getState("blay_waystones:state") === "attuned") {
        dropContainedShard(block, block.permutation);
        block.setPermutation(block.permutation.withState("blay_waystones:state", "empty"));
    }
}

//# sourceMappingURL=../../_blay_waystonesDebug/warp_plates.js.map
