import { BlockPermutation, Direction, MolangVariableMap, system, world, } from "@minecraft/server";
import { getDirectionFromVectorH, getDistance, isAirOrReplaceable, vectorCenterH, vectorNormalize, vectorAdd, vectorRotateY, vectorSubtract, isTeleportAllowed, } from "./utils";
import { getWaystoneForBlock, getWaystoneForId, getWaystoneType, WaystoneType } from "./waystone_registry";
import { playWarpPortalTeleportEffects } from "./teleport_effects";
import { teleportToWaystone } from "./teleporter";
export class WarpPortalEmitterComponent {
    onTick(event) {
        spawnIdleParticles(event.block);
    }
}
export class WarpPortalComponent {
    onStepOn(event) {
        const warpPortal = getOrCreateWarpPortal(event.block);
        const boundTo = getBlockBoundTo(event.block);
        if (warpPortal &&
            boundTo &&
            event.entity &&
            !isWarpPortalEntityCooldown(event.entity) &&
            isTeleportAllowed(event.entity, boundTo)) {
            const dist = getDistance(warpPortal.pos, boundTo.pos);
            playWarpPortalTeleportEffects(event.entity, warpPortal, boundTo);
            if (warpPortal.dimensionId !== boundTo.dimensionId || dist > 1) {
                teleportToWaystone(event.entity, boundTo);
            }
            storeReturnPortal(event.entity, warpPortal, boundTo);
        }
        else if (!boundTo) {
            event.block.dimension.spawnParticle("blay_waystones:warp_portal_close", vectorCenterH(event.block.location));
            event.block.dimension.playSound("random.fizz", event.block.location, {
                volume: 0.25,
            });
            if (event.entity?.typeId === "minecraft:player") {
                event.entity.sendMessage({ translate: "chat.blay_waystones.portal_attunement_lost" });
            }
            removeWarpPortalAt(event.block);
        }
    }
    onStepOff(event) {
        if (event.entity) {
            clearWarpPortalEntityCooldown(event.entity);
        }
    }
    onTick(event) {
        event.block.dimension.spawnParticle("blay_waystones:warp_portal_close", vectorCenterH(event.block.location));
        event.block.dimension.playSound("random.fizz", event.block.location, {
            volume: 0.25,
        });
        removeWarpPortalAt(event.block);
    }
}
function getOrCreateWarpPortal(block) {
    return getWaystoneForBlock(block);
}
function getBlockBoundTo(block) {
    const warpPortal = getOrCreateWarpPortal(block);
    if (warpPortal) {
        const warpPortalKey = getWarpPortalBoundToKey(warpPortal.dimensionId, warpPortal.pos);
        const waystoneId = world.getDynamicProperty(warpPortalKey);
        if (typeof waystoneId === "string") {
            return getWaystoneForId(waystoneId);
        }
    }
    return undefined;
}
function setBlockBoundTo(block, waystone) {
    const warpPortal = getOrCreateWarpPortal(block);
    if (warpPortal) {
        const warpPortalKey = getWarpPortalBoundToKey(warpPortal.dimensionId, warpPortal.pos);
        if (waystone) {
            world.setDynamicProperty(warpPortalKey, waystone.id);
        }
        else {
            world.setDynamicProperty(warpPortalKey, undefined);
        }
    }
}
function hasAdjacentPortal(block) {
    const adjacentOffsets = [
        { x: 1, y: 0, z: 0 },
        { x: -1, y: 0, z: 0 },
        { x: 0, y: 0, z: 1 },
        { x: 0, y: 0, z: -1 },
        { x: 1, y: 0, z: 1 },
        { x: -1, y: 0, z: -1 },
        { x: 1, y: 0, z: -1 },
        { x: -1, y: 0, z: 1 },
    ];
    for (const offset of adjacentOffsets) {
        const adjacentPos = {
            x: block.location.x + offset.x,
            y: block.location.y + offset.y,
            z: block.location.z + offset.z,
        };
        const adjacentBlock = block.dimension.getBlock(adjacentPos);
        if (adjacentBlock && adjacentBlock.hasTag("blay_waystones:warp_portal")) {
            return true;
        }
    }
    return false;
}
function isViewObstructed(entity, block) {
    const start = vectorAdd(entity.location, { x: 0, y: 1, z: 0 });
    const end = vectorAdd(block.location, { x: 0, y: 1, z: 0 });
    const dir = { x: end.x - start.x, y: 0, z: end.z - start.z };
    const dist = Math.sqrt(dir.x * dir.x + dir.y * dir.y + dir.z * dir.z);
    try {
        const obstacle = entity.dimension.getBlockFromRay(start, dir, {
            maxDistance: dist,
        });
        if (obstacle && !isAirOrReplaceable(obstacle.block)) {
            return true;
        }
    }
    catch (ignored) { }
    return false;
}
function getRelativeTopmostBlock(dimension, pos) {
    let groundBlock = dimension.getTopmostBlock(pos);
    if (groundBlock && Math.abs(groundBlock.location.y - pos.y) <= 5) {
        return groundBlock;
    }
    groundBlock = dimension.getBlock(pos);
    while (groundBlock && isAirOrReplaceable(groundBlock)) {
        try {
            groundBlock = groundBlock?.below();
        }
        catch (e) {
            break;
        }
    }
    let blockAbove = groundBlock?.above();
    while (blockAbove && !isAirOrReplaceable(blockAbove)) {
        groundBlock = blockAbove;
        try {
            blockAbove = blockAbove?.above();
        }
        catch (e) {
            break;
        }
    }
    return groundBlock;
}
function findSpaceForWarpPortalInFront(entity) {
    const dimension = entity.dimension;
    const center = entity.location;
    const dir = entity.getViewDirection();
    const minDistance = 3;
    const maxDistance = 4;
    const coneAngle = Math.PI / 6;
    const steps = 3;
    const candidates = [];
    for (let dist = minDistance; dist <= maxDistance; dist++) {
        for (let step = -steps; step <= steps; step++) {
            const angle = (step / steps) * coneAngle;
            const rotatedDir = vectorRotateY(dir, angle);
            const targetPos = vectorAdd(center, {
                x: rotatedDir.x * dist,
                y: 0,
                z: rotatedDir.z * dist,
            });
            const groundBlock = getRelativeTopmostBlock(dimension, targetPos);
            const block = groundBlock?.above();
            const blockAbove = block?.above();
            if (block &&
                blockAbove &&
                isLegalPortalGround(block.below()) &&
                isAirOrReplaceable(block) &&
                isAirOrReplaceable(blockAbove) &&
                isAirOrReplaceable(blockAbove.above()) &&
                !hasAdjacentPortal(block) &&
                !isViewObstructed(entity, block)) {
                candidates.push(block.location);
            }
        }
    }
    if (candidates.length) {
        // Find the candidate closest to the player's forward direction
        let bestCandidate = candidates[0];
        let bestDistance = Infinity;
        for (const candidate of candidates) {
            const candidateDir = vectorNormalize(vectorSubtract(candidate, center));
            const alignment = Math.abs(candidateDir.x - dir.x) + Math.abs(candidateDir.z - dir.z);
            if (alignment < bestDistance) {
                bestDistance = alignment;
                bestCandidate = candidate;
            }
        }
        return bestCandidate;
    }
    return undefined;
}
function isLegalPortalGround(block) {
    if (block?.hasTag("blay_waystones:no_portals_on_top")) {
        return false;
    }
    const illegalBlocks = [
        "minecraft:rail",
        "minecraft:activator_rail",
        "minecraft:detector_rail",
        "minecraft:golden_rail",
    ];
    for (const illegalBlock of illegalBlocks) {
        if (block?.matches(illegalBlock)) {
            return false;
        }
    }
    return true;
}
function spawnWarpPortalAt(dimensionId, pos, target, facing) {
    const dimension = world.getDimension(dimensionId);
    const block = dimension.getBlock(pos);
    const blockAbove = block?.above();
    if (block && blockAbove && isAirOrReplaceable(block) && isAirOrReplaceable(blockAbove)) {
        block.setPermutation(BlockPermutation.resolve("blay_waystones:warp_portal", {
            "blay_waystones:half": "bottom",
            "minecraft:cardinal_direction": facing.toLowerCase(),
        }));
        blockAbove.setPermutation(BlockPermutation.resolve("blay_waystones:warp_portal", {
            "blay_waystones:half": "top",
            "minecraft:cardinal_direction": facing.toLowerCase(),
        }));
        dimension.spawnParticle("blay_waystones:warp_portal_open", vectorAdd(block.location, { x: 0.5, y: 1, z: 0.5 }));
        spawnIdleParticles(block);
        spawnIdleParticles(blockAbove);
        dimension.playSound("portal.travel", block.location, {
            volume: 0.25,
        });
        setBlockBoundTo(block, target);
    }
}
export function hasSpaceForWarpPortal(entity) {
    return !!findSpaceForWarpPortalInFront(entity);
}
export function spawnWarpPortalAround(entity, target) {
    const pos = findSpaceForWarpPortalInFront(entity);
    if (pos) {
        const facing = getDirectionFromVectorH(entity.getViewDirection());
        spawnWarpPortalAt(entity.dimension.id, pos, target, facing ?? Direction.North);
        return true;
    }
    return false;
}
export function removeWarpPortalAt(block) {
    const blockAbove = block.above();
    if (block?.matches("blay_waystones:warp_portal")) {
        block.setPermutation(BlockPermutation.resolve("minecraft:air"));
    }
    if (blockAbove?.matches("blay_waystones:warp_portal")) {
        blockAbove.setPermutation(BlockPermutation.resolve("minecraft:air"));
    }
    const warpPlateKey = getWarpPortalBoundToKey(block.dimension.id, block.location);
    world.setDynamicProperty(warpPlateKey, undefined);
}
const warpPortalBoundToKeyPrefix = "blay_waystones:warp_portal:";
export function getWarpPortalBoundToKey(dimensionId, pos) {
    return warpPortalBoundToKeyPrefix + dimensionId + "," + pos.x + "," + pos.y + "," + pos.z;
}
export function storeReturnPortal(entity, warpPortal, boundTo) {
    entity.setDynamicProperty("blay_waystones:return_portal", JSON.stringify({
        originId: boundTo.id,
        dimensionId: warpPortal.dimensionId,
        pos: warpPortal.pos,
        expires: system.currentTick + 5 * 60 * 20,
    }));
}
export function getReturnPortal(entity) {
    const returnPortal = entity.getDynamicProperty("blay_waystones:return_portal");
    if (typeof returnPortal === "string") {
        const returnPortalData = JSON.parse(returnPortal);
        const origin = getWaystoneForId(returnPortalData.originId);
        const isExpired = returnPortalData.expires <= system.currentTick;
        if (origin && !isExpired) {
            return {
                id: "return_portal",
                dimensionId: returnPortalData.dimensionId,
                pos: returnPortalData.pos,
                data: {
                    type: WaystoneType.WarpPortal,
                },
            };
        }
    }
    return undefined;
}
const entityCooldownKey = "blay_waystones:warp_portal_cooldown";
function isWarpPortalEntityCooldown(entity) {
    return !!entity.getDynamicProperty(entityCooldownKey);
}
export function setWarpPortalEntityCooldown(entity, warpPortal) {
    entity.setDynamicProperty(entityCooldownKey, JSON.stringify({
        dimensionId: warpPortal.dimensionId,
        pos: warpPortal.pos,
    }));
}
function clearWarpPortalEntityCooldown(entity) {
    entity.setDynamicProperty(entityCooldownKey, undefined);
}
function spawnIdleParticles(block) {
    const facing = block.permutation.getState("minecraft:cardinal_direction");
    const vars = new MolangVariableMap();
    if (facing === "north" || facing === "south") {
        vars.setVector3("box_size", {
            x: 0.4,
            y: 1,
            z: 0.1,
        });
    }
    else {
        vars.setVector3("box_size", {
            x: 0.1,
            y: 1,
            z: 0.4,
        });
    }
    block.dimension.spawnParticle("blay_waystones:warp_portal", vectorCenterH(block.location), vars);
}
export function verifyWarpPortal(player, waystone, retry = true) {
    if (!player.isValid()) {
        return;
    }
    // This is only for cleaning up portals that were removed from the world through unexpected means like external world editors or pistons.
    const dimension = world.getDimension(waystone.dimensionId);
    if (dimension) {
        const block = dimension.getBlock(waystone.pos);
        if (block) {
            const type = getWaystoneType(block);
            if (type !== waystone.data.type) {
                player.setDynamicProperty("blay_waystones:return_portal", undefined);
            }
        }
        else if (retry) {
            // There is no way to trigger exactly when a chunk is loaded, so we just check five seconds after. Not great but ok given this is just edge case handling.
            system.runTimeout(() => verifyWarpPortal(player, waystone, false), 100);
        }
    }
}

//# sourceMappingURL=../../_blay_waystonesDebug/warp_portals.js.map
