import { system, world } from "@minecraft/server";
import { parseDirection, randomUUID } from "./utils";
import { removeWaystoneActivations } from "./player_persistence";
import { GROUP_THE_END, GROUP_THE_NETHER, GROUP_VILLAGES } from "./waystone_groups";
const waystoneIdKeyPrefix = "blay_waystones:id:";
const waystonePosKeyPrefix = "blay_waystones:pos:";
export function isWaystoneIdKey(propertyId) {
    return propertyId.startsWith(waystoneIdKeyPrefix);
}
export function getWaystoneIdKey(id) {
    return waystoneIdKeyPrefix + id;
}
export function getWaystoneIdFromKey(idKey) {
    return idKey.substring(waystoneIdKeyPrefix.length);
}
export function getWaystonePosKey(dimensionId, pos) {
    return waystonePosKeyPrefix + dimensionId + "," + pos.x + "," + pos.y + "," + pos.z;
}
export var WaystoneType;
(function (WaystoneType) {
    WaystoneType["Waystone"] = "waystone";
    WaystoneType["AmethystSharestone"] = "amethyst_sharestone";
    WaystoneType["CopperSharestone"] = "copper_sharestone";
    WaystoneType["DiamondSharestone"] = "diamond_sharestone";
    WaystoneType["EmeraldSharestone"] = "emerald_sharestone";
    WaystoneType["GoldSharestone"] = "gold_sharestone";
    WaystoneType["LapisSharestone"] = "lapis_sharestone";
    WaystoneType["PrismarineSharestone"] = "prismarine_sharestone";
    WaystoneType["RedstoneSharestone"] = "redstone_sharestone";
    WaystoneType["WarpPlate"] = "warp_plate";
    WaystoneType["WarpPortal"] = "warp_portal";
})(WaystoneType || (WaystoneType = {}));
export function isSharestoneType(type) {
    return (type === WaystoneType.AmethystSharestone ||
        type === WaystoneType.CopperSharestone ||
        type === WaystoneType.DiamondSharestone ||
        type === WaystoneType.EmeraldSharestone ||
        type === WaystoneType.GoldSharestone ||
        type === WaystoneType.LapisSharestone ||
        type === WaystoneType.PrismarineSharestone ||
        type === WaystoneType.RedstoneSharestone);
}
export function getWaystoneType(block) {
    if (!block) {
        return undefined;
    }
    if (block.hasTag("blay_waystones:waystone")) {
        return WaystoneType.Waystone;
    }
    else if (block.hasTag("blay_waystones:sharestone")) {
        if (block.permutation.matches("blay_waystones:amethyst_sharestone")) {
            return WaystoneType.AmethystSharestone;
        }
        else if (block.permutation.matches("blay_waystones:copper_sharestone")) {
            return WaystoneType.CopperSharestone;
        }
        else if (block.permutation.matches("blay_waystones:diamond_sharestone")) {
            return WaystoneType.DiamondSharestone;
        }
        else if (block.permutation.matches("blay_waystones:emerald_sharestone")) {
            return WaystoneType.EmeraldSharestone;
        }
        else if (block.permutation.matches("blay_waystones:gold_sharestone")) {
            return WaystoneType.GoldSharestone;
        }
        else if (block.permutation.matches("blay_waystones:lapis_sharestone")) {
            return WaystoneType.LapisSharestone;
        }
        else if (block.permutation.matches("blay_waystones:prismarine_sharestone")) {
            return WaystoneType.PrismarineSharestone;
        }
        else if (block.permutation.matches("blay_waystones:redstone_sharestone")) {
            return WaystoneType.RedstoneSharestone;
        }
    }
    else if (block.hasTag("blay_waystones:warp_plate")) {
        return WaystoneType.WarpPlate;
    }
    else if (block.hasTag("blay_waystones:warp_portal")) {
        return WaystoneType.WarpPortal;
    }
    return undefined;
}
function populateFromBlock(descriptor, block) {
    const facing = parseDirection(block.permutation.getState("minecraft:cardinal_direction"));
    if (facing) {
        descriptor.data.facing = facing;
    }
    descriptor.data.type = getWaystoneType(block);
    if (!descriptor.data.group) {
        if (block.permutation.getState("blay_waystones:origin") === "village") {
            descriptor.data.group = GROUP_VILLAGES;
        }
        else if (block.dimension.id === "minecraft:the_end") {
            descriptor.data.group = GROUP_THE_END;
        }
        else if (block.dimension.id === "minecraft:nether") {
            descriptor.data.group = GROUP_THE_NETHER;
        }
    }
}
export function getWaystoneForBlock(block) {
    let effectiveBlock = block;
    if (block?.permutation.getState("blay_waystones:half") === "top") {
        effectiveBlock = block.below();
    }
    const type = getWaystoneType(effectiveBlock);
    if (!effectiveBlock || !type) {
        return undefined;
    }
    const descriptor = getWaystoneForPos(effectiveBlock.dimension.id, effectiveBlock, true);
    if (descriptor) {
        populateFromBlock(descriptor, effectiveBlock);
    }
    return descriptor;
}
export function getWaystoneForId(id) {
    const idKey = getWaystoneIdKey(id);
    const waystoneDataJson = world.getDynamicProperty(idKey);
    if (typeof waystoneDataJson === "string") {
        const waystoneData = JSON.parse(waystoneDataJson);
        if (waystoneData && waystoneData.pos && waystoneData.dimensionId) {
            return {
                id: id,
                pos: waystoneData.pos,
                dimensionId: waystoneData.dimensionId,
                data: waystoneData,
            };
        }
    }
    return undefined;
}
export function getWaystoneForPos(dimensionId, pos, createIfMissing = false) {
    const posKey = getWaystonePosKey(dimensionId, pos);
    let waystoneId = world.getDynamicProperty(posKey);
    if (typeof waystoneId !== "string") {
        waystoneId = undefined;
    }
    let waystoneData = {
        pos: { x: pos.x, y: pos.y, z: pos.z },
        dimensionId: dimensionId,
    };
    if (waystoneId) {
        const idKey = getWaystoneIdKey(waystoneId);
        const waystoneDataJson = world.getDynamicProperty(idKey);
        if (typeof waystoneDataJson === "string") {
            waystoneData = JSON.parse(waystoneDataJson);
        }
    }
    else if (createIfMissing) {
        waystoneId = randomUUID();
    }
    else {
        return undefined;
    }
    return { id: waystoneId, pos: { x: pos.x, y: pos.y, z: pos.z }, dimensionId: dimensionId, data: waystoneData };
}
export function removeWaystone(waystone) {
    world.setDynamicProperty(getWaystoneIdKey(waystone.id), undefined);
    world.setDynamicProperty(getWaystonePosKey(waystone.dimensionId, waystone.pos), undefined);
    removeWaystoneActivations(waystone);
    return waystone;
}
export function removeWaystoneAt(dimensionId, pos) {
    const waystone = getWaystoneForPos(dimensionId, pos);
    if (waystone) {
        return removeWaystone(waystone);
    }
}
export function saveWaystoneData(waystone) {
    const idKey = getWaystoneIdKey(waystone.id);
    world.setDynamicProperty(idKey, JSON.stringify(waystone.data));
    world.setDynamicProperty(getWaystonePosKey(waystone.dimensionId, waystone.pos), waystone.id);
}
export function verifyWaystoneBlock(waystone, retry = true) {
    // This is only for cleaning up waystones that were removed from the world through unexpected means like external world editors or pistons.
    const dimension = world.getDimension(waystone.dimensionId);
    if (dimension) {
        const block = dimension.getBlock(waystone.pos);
        if (block) {
            const type = getWaystoneType(block);
            if (type !== waystone.data.type) {
                removeWaystone(waystone);
            }
        }
        else if (retry) {
            // There is no way to trigger exactly when a chunk is loaded, so we just check five seconds after. Not great but ok given this is just edge case handling.
            system.runTimeout(() => verifyWaystoneBlock(waystone, false), 100);
        }
    }
}

//# sourceMappingURL=../../_blay_waystonesDebug/waystone_registry.js.map
