import { world, } from "@minecraft/server";
import { teleportToWaystone } from "./teleporter";
import { vectorCenterH } from "./utils";
import { getWaystoneForBlock, removeWaystoneAt, saveWaystoneData } from "./waystone_registry";
import { tryBindBlankScroll } from "./scrolls";
import { showWaystoneSetup } from "./ui/ui_waystone_setup";
import { showWaystoneSelection } from "./ui/ui_waystone_selection";
import { getActivatedWaystones, isWaystoneActivated, saveWaystoneActivation } from "./player_persistence";
import { playRegularTeleportEffects } from "./teleport_effects";
import { generateName } from "./name_generator";
export class WaystoneTopComponent {
    onPlayerInteract(event) {
        const player = event.player;
        if (player) {
            // The waystone is registered to the base of the waystone double block, so we delegate
            let baseBlock = event.block?.below();
            if (baseBlock?.hasTag("blay_waystones:waystone")) {
                const waystone = getWaystoneForBlock(baseBlock);
                if (waystone) {
                    interactWaystone(player, waystone);
                }
            }
        }
    }
    onPlayerDestroy(event) {
        destroyWaystoneBase(event.dimension, event.block.below());
    }
}
export class WaystoneComponent {
    beforeOnPlayerPlace(event) {
        // Only allow placement if the block above is free
        const above = event.block.above();
        if (!above?.isAir) {
            event.cancel = true;
        }
    }
    onPlace(event) {
        const above = event.block.above();
        if (above?.isAir) {
            above.setPermutation(event.block.permutation.withState("blay_waystones:half", "top"));
        }
    }
    // Post-placement behaviour is done in main due to missing player in BlockComponentPlaceEvent
    onPlayerInteract(event) {
        const player = event.player;
        const waystone = getWaystoneForBlock(event.block);
        if (player && waystone) {
            interactWaystone(player, waystone);
        }
    }
    onPlayerDestroy(event) {
        removeWaystoneAt(event.dimension.id, event.block.location);
    }
}
function activateWaystone(player, waystone) {
    player.dimension.runCommandAsync("recipe give " + player.name + " blay_waystones:blank_scroll");
    player.dimension.runCommandAsync("recipe give " + player.name + " blay_waystones:return_scroll");
    player.dimension.runCommandAsync("recipe give " + player.name + " blay_waystones:warp_scroll");
    player.dimension.runCommandAsync("recipe give " + player.name + " blay_waystones:portal_scroll");
    player.dimension.runCommandAsync("recipe give " + player.name + " blay_waystones:warp_stone");
    const wasAlreadyActivated = isWaystoneActivated(player, waystone);
    if (!wasAlreadyActivated) {
        // If the waystone doesn't have a name yet (e.g. spawned in a village), generate one for it
        if (!waystone.data.name) {
            waystone.data.name = generateName();
            saveWaystoneData(waystone);
        }
        player.dimension.spawnParticle("blay_waystones:waystone_activation", vectorCenterH(waystone.pos));
        player.sendMessage({
            translate: "chat.blay_waystones.waystone_activated",
            with: [waystone.data.name],
        });
    }
    saveWaystoneActivation(player, waystone);
    markVillageWaystoneActivated(waystone);
    return !wasAlreadyActivated;
}
function markVillageWaystoneActivated(waystone) {
    const dimension = world.getDimension(waystone.dimensionId);
    const block = dimension.getBlock(waystone.pos);
    if (!block) {
        return;
    }
    let topBlock = null;
    if (block.permutation.getState("blay_waystones:half") === "top") {
        topBlock = block;
    }
    else {
        topBlock = block.above();
        if (topBlock?.permutation.getState("blay_waystones:half") !== "top") {
            topBlock = null;
        }
    }
    if (topBlock && topBlock.permutation.getState("blay_waystones:activated") === false) {
        topBlock.setPermutation(topBlock.permutation.withState("blay_waystones:activated", true));
    }
}
export async function interactWaystone(player, waystone) {
    if (activateWaystone(player, waystone)) {
        return;
    }
    // Block interaction takes precedence, so we have to trigger scroll binding from here
    if (tryBindBlankScroll(player, waystone)) {
        return;
    }
    if (player.isSneaking) {
        showWaystoneSetup(player, waystone);
    }
    else {
        const selectedWaystone = await showWaystoneSelection(player, getActivatedWaystones(player), {
            fromWaystone: waystone,
            noTargetsAvailableMessage: {
                translate: "chat.blay_waystones.waystone_no_waystones_available",
            },
        });
        if (selectedWaystone && player.isValid()) {
            playRegularTeleportEffects(player, selectedWaystone);
            teleportToWaystone(player, selectedWaystone);
        }
    }
}
export function destroyWaystoneBase(dimension, block) {
    if (block?.hasTag("blay_waystones:waystone")) {
        dimension.runCommand("setblock " + block.x + " " + block.y + " " + block.z + " air destroy");
        removeWaystoneAt(dimension.id, block.location);
    }
}

//# sourceMappingURL=../../_blay_waystonesDebug/waystones.js.map
