import { system, world } from "@minecraft/server";
import { getDistance, getDistanceH, isAirOrReplaceable, randomUUID } from "./utils";
class VillageWaystoneGenerator {
    constructor() {
        this.areaSize = 5;
        this.clearanceSize = this.areaSize + 2;
        this.blockCache = new Map();
    }
    posToString(pos) {
        return `${pos.x},${pos.z}`;
    }
    getCachedBlock(dimension, pos) {
        const key = this.posToString(pos);
        if (this.blockCache.has(key)) {
            return this.blockCache.get(key);
        }
        try {
            const block = dimension.getTopmostBlock(pos);
            if (block && block.isValid()) {
                this.blockCache.set(key, block);
                return block;
            }
        }
        catch (e) { }
        return undefined;
    }
    isPath(dimension, pos) {
        const block = this.getCachedBlock(dimension, pos);
        if (!block) {
            return false;
        }
        const validPaths = ["minecraft:grass_path", "minecraft:smooth_sandstone"];
        for (const validPath of validPaths) {
            if (block.matches(validPath)) {
                return true;
            }
        }
        return false;
    }
    isValidGround(dimension, pos) {
        const block = this.getCachedBlock(dimension, pos);
        if (!block) {
            return false;
        }
        const validGrounds = ["minecraft:grass_block", "minecraft:dirt", "minecraft:sand"];
        for (const validGround of validGrounds) {
            if (block.matches(validGround)) {
                return true;
            }
        }
        return false;
    }
    isReplaceable(dimension, pos) {
        return isAirOrReplaceable(dimension.getBlock(pos));
    }
    *findNearbyPath(dimension, startPosition, maxDistance = 25) {
        const queue = [startPosition];
        const visited = new Set();
        visited.add(this.posToString(startPosition));
        while (queue.length > 0) {
            const position = queue.shift();
            if (this.isPath(dimension, position)) {
                return dimension.getTopmostBlock(position)?.location;
            }
            const directions = [
                { x: 1, z: 0 },
                { x: -1, z: 0 },
                { x: 0, z: 1 },
                { x: 0, z: -1 },
            ];
            for (const direction of directions) {
                const newPosition = { x: position.x + direction.x, y: 0, z: position.z + direction.z };
                const key = this.posToString(newPosition);
                const dist = getDistanceH(startPosition, newPosition);
                if (!visited.has(key) && dist <= maxDistance) {
                    visited.add(key);
                    queue.push(newPosition);
                }
            }
            yield;
        }
    }
    *findConnectingClearGround(dimension, pathPos) {
        const directions = [
            { x: 1, z: 0 },
            { x: -1, z: 0 },
            { x: 0, z: 1 },
            { x: 0, z: -1 },
        ];
        withNextDirection: for (const direction of directions) {
            let currentY = null;
            const halfSize = Math.floor(this.clearanceSize / 2);
            const areaOffset = {
                x: pathPos.x + direction.x * (1 + halfSize),
                y: pathPos.y,
                z: pathPos.z + direction.z * (1 + halfSize),
            };
            for (let x = -halfSize; x <= halfSize; x++) {
                for (let z = -halfSize; z <= halfSize; z++) {
                    const areaPos = {
                        x: areaOffset.x + x,
                        y: areaOffset.y,
                        z: areaOffset.z + z,
                    };
                    const block = this.getCachedBlock(dimension, areaPos);
                    if (currentY === null) {
                        currentY = block?.y ?? null;
                    }
                    else if (currentY !== block?.y) {
                        yield;
                        continue withNextDirection;
                    }
                    if (!this.isValidGround(dimension, areaPos)) {
                        yield;
                        continue withNextDirection;
                    }
                }
            }
            // Check if all blocks within the structure area are replaceable
            for (let x = -halfSize; x <= halfSize; x++) {
                for (let y = 1; y <= this.areaSize; y++) {
                    for (let z = -halfSize; z <= halfSize; z++) {
                        const areaPos = {
                            x: areaOffset.x + x,
                            y: areaOffset.y + y,
                            z: areaOffset.z + z,
                        };
                        if (!this.isReplaceable(dimension, areaPos)) {
                            yield;
                            continue withNextDirection;
                        }
                    }
                }
            }
            return { x: areaOffset.x, y: currentY ?? areaOffset.y, z: areaOffset.z };
        }
        return undefined;
    }
    *findConnectingClearGroundAlongPath(dimension, startPosition) {
        const queue = [startPosition];
        const visited = new Set();
        visited.add(this.posToString(startPosition));
        while (queue.length > 0) {
            const position = dimension.getTopmostBlock(queue.shift())?.location;
            if (!position) {
                continue;
            }
            const clearArea = yield* this.findConnectingClearGround(dimension, position);
            if (clearArea) {
                return clearArea;
            }
            const directions = [
                { x: 1, z: 0 },
                { x: -1, z: 0 },
                { x: 0, z: 1 },
                { x: 0, z: -1 },
            ];
            for (const direction of directions) {
                const newPosition = { x: position.x + direction.x, y: 0, z: position.z + direction.z };
                const key = this.posToString(newPosition);
                if (!visited.has(key) && this.isPath(dimension, newPosition)) {
                    visited.add(key);
                    queue.push(newPosition);
                }
            }
            yield;
        }
    }
    determineWaystoneStructure(dimension, pos) {
        const ground = this.getCachedBlock(dimension, pos);
        if (ground?.permutation.matches("minecraft:sand")) {
            return "blay_waystones:sandy_waystone";
        }
        return "blay_waystones:waystone";
    }
    *generateInner(dimension, pos) {
        const pathPos = yield* this.findNearbyPath(dimension, pos);
        if (!pathPos) {
            return false;
        }
        const clearGround = yield* this.findConnectingClearGroundAlongPath(dimension, pathPos);
        if (!clearGround) {
            return false;
        }
        const halfSize = Math.floor(this.areaSize / 2);
        const structure = this.determineWaystoneStructure(dimension, clearGround);
        world.structureManager.place(structure, dimension, {
            x: clearGround.x - halfSize,
            y: clearGround.y + 1,
            z: clearGround.z - halfSize,
        });
        return true;
    }
    *generate(dimension, pos) {
        const iterator = this.generateInner(dimension, pos);
        let result = iterator.next();
        while (!result.done) {
            yield;
            result = iterator.next();
        }
    }
}
function getVillageId(pos) {
    const villagesJson = world.getDynamicProperty("blay_waystones:villages");
    if (typeof villagesJson === "string") {
        const villages = JSON.parse(villagesJson);
        for (const villageId of Object.keys(villages)) {
            const distance = getDistance(pos, villages[villageId]);
            if (distance < 250) {
                return villageId;
            }
        }
    }
    return null;
}
function storageVillageId(pos, id) {
    let villages = {};
    const villagesJson = world.getDynamicProperty("blay_waystones:villages");
    if (typeof villagesJson === "string") {
        villages = JSON.parse(villagesJson);
    }
    villages[id] = pos;
    world.setDynamicProperty("blay_waystones:villages", JSON.stringify(villages));
}
export function tryVillageWaystoneGeneration(dimension, pos) {
    const existingVillageId = getVillageId(pos);
    if (existingVillageId) {
        return;
    }
    const villageId = randomUUID();
    storageVillageId(pos, villageId);
    const generator = new VillageWaystoneGenerator();
    system.runJob(generator.generate(dimension, pos));
}

//# sourceMappingURL=../../_blay_waystonesDebug/worldgen.js.map
